/*
 *	carbon.h
 *
 *	The carbon copy class can be used to replicated easily any OPaC data type
 *	by serializing it once and then deserializing it as many times as needed.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_CARBON_H_
#define	_OPaC_CARBON_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (carbon.h);

/****************************************************************/

/*
 *	A carbon copy object can be used to store data once
 *	and retrieve it many times.
 */

class OP_PUBLIC OPCarbonCopy : public OPStorage
{
	
private:
	struct DataBlock
	{
		DataBlock*		next;					//	next data block (if any)
		Card8*			data;					//	data stored in block
		Card32			len;					//	current length of stored data
		Card32			max;					//	maximum possible length in this block
	};
	
private:
	DataBlock*			block_head;				//	first data block
	DataBlock*			block_tail;				//	last data block

	DataBlock*			current_block;			//	current block
	Card32				current_offset;			//	current offset in block
	
protected:
	OPCarbonCopy ();
	virtual ~ OPCarbonCopy ();
	
	virtual void StoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	
	virtual void AddBlock (Card32 size);

public:
	virtual void StartStoring ();
	virtual void StartRestoring ();
	
	//	The carbon copy has to handle the raw data input/output, the other data
	//	types are being handled by the super-class.
	
	virtual void StoreData (const void* data, Card32 size);	/* OVERRIDDEN */
	virtual void RestoreData (void* data, Card32 size);		/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPCarbonCopy, OPStorage);
};

/****************************************************************/

#endif /* _OPaC_CARBON_H_ */
