/*
 *	compiler.h
 *
 *	This file contains the compiler dependent type definitions. It is compiler
 *	and machine specific.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_COMPILER_H_
#define	_OPaC_COMPILER_H_

#include <stddef.h>

#ifdef __GNUC__

#define	OP_INTERFACE(n)							\
/*	#pragma interface	*/						\
	class OPObject

#define	OP_IMPLEMENTATION(n)					\
/*	#pragma implementation	*/					\
	class OPObject

#else

#define	OP_INTERFACE(n)							\
	class OPObject

#define	OP_IMPLEMENTATION(n)					\
	class OPObject

#endif

/*
 *	Miscellanous integer definitions. Only these types should be
 *	used where the size is important.
 */

#if defined(WIN32)

typedef	signed __int8		Int8;				//	 8-bit signed
typedef	signed __int16		Int16;				//	16-bit signed
typedef	signed __int32		Int32;				//	32-bit signed
typedef signed __int64		Int64;				//	64-bit signed

typedef	unsigned __int8		Card8;				//	 8-bit unsigned
typedef	unsigned __int16	Card16;				//	16-bit unsigned
typedef	unsigned __int32	Card32;				//	32-bit unsigned
typedef unsigned __int64	Card64;				//	64-bit unsigned

#ifdef __BUILD_OPAC_DLL__
#define	OP_PUBLIC		__declspec(dllexport)
#else
#define OP_PUBLIC		__declspec(dllimport)
#endif

#pragma warning (disable : 4003)				//	no warning for empty args in macros
#pragma warning (disable : 4181)				//	no warning for const applied to typedef

#else

typedef	signed char			Int8;				//	 8-bit signed
typedef	signed short		Int16;				//	16-bit signed
typedef	signed long			Int32;				//	32-bit signed
typedef signed long long	Int64;				//	64-bit signed

typedef	unsigned char		Card8;				//	 8-bit unsigned
typedef	unsigned short		Card16;				//	16-bit unsigned
typedef	unsigned long		Card32;				//	32-bit unsigned
typedef unsigned long long	Card64;				//	64-bit unsigned

#endif

#ifndef OP_PUBLIC
#define	OP_PUBLIC
#endif

#ifndef OP_APP
#define	OP_APP
#endif

typedef double				Real;				//	real number (quite precise)


/*
 *	Other compiler specific definitions :
 */

#define	OP_USE(x)	do { void* _u_ = (void*)(&x); _u_ = 0; } while (0)

#include <setjmp.h>
#include <malloc.h>
#include <stdlib.h>
typedef jmp_buf			JumpBuffer;			//	buffer for setjmp
#define	stack_alloc(s)	alloca((size_t)(s))

/****************************************************************/

#endif /* _OPaC_COMPILER_H_ */
