/*
 *	led7disp.h
 *
 *	The LED 7-segment Display class. This can be used to display
 *	numerical values.
 *
 *	(C) Copyright 1994-1997, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_LED7DISP_H_
#define	_OPaC_LED7DISP_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (texttag.h);

#ifndef _OPaC_TEXTTAG_H_
#include <OPaC/texttag.h>
#endif

/****************************************************************/

/*
 *	The LED 7-segment Display class.
 */

class OP_PUBLIC OPLed7Display : public OPTextTagBase
{

protected:
	Count				led_digits;			//	number of digits (0 => dynamic)
	Bool				led_signed;			//	TRUE => signed value (prefixed by "-")
	OPColorPixel		led_color;			//	color of text
	
protected:
	OPLed7Display ();
	virtual ~ OPLed7Display ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual Bool SetDigitsSign (Count digits, Bool use_sign);
	virtual Bool SetDigits (Count digits);
	virtual Bool SetSign (Bool use_sign);
	virtual Bool SetColor (OPColorPixel pixel);
	
	virtual Count ReturnDigits () const;
	virtual Bool ReturnSigned () const;
	virtual OPColorPixel ReturnColor () const;
	
	//	General usage methods :
	
	virtual void GetMinSize (OPCoordOut dx, OPCoordOut dy) const;/* OVERRIDDEN */
	virtual void GetMaxSize (OPCoordOut dx, OPCoordOut dy) const;/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */

	OP_DEFINE_CLASS (OPLed7Display, OPTextTagBase);
};

/****************************************************************/

#endif /* _OPaC_LED7DISP_H_ */
