/*
 *	message.h
 *
 *	This file defines the message class.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_MESSAGE_H_
#define	_OPaC_MESSAGE_H_

#ifndef _OPaC_OBJECT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_ATOM_H_
#include <OPaC/atom.h>
#endif

OP_INTERFACE (message.h);

/****************************************************************/

/*
 *	A message is a generic data holder used to exchange
 *	data from one object (the sender) to another object
 *	(the receiver).
 *
 *	The data can be stored into the message in the same
 *	fashion as in an archive. It will then be retrieved
 *	in the same order it has been stored (FIFO).
 */

class OP_PUBLIC OPMessage : public OPStorage
{
	
private:
	Card8*				data_ptr;				//	current pointer to data
	Card32				data_len;				//	remaining free room in heap
	Card8				data_heap[256];			//	heap to store data
	
protected:
	Bool				target_ok;				//	TRUE => use tight message passing
	Bool				method_ok;				//	TRUE => use lose message passing
	
	OPTargetRef			target;					//	target description
	OPAtomRef			method;					//	method name
	
protected:
	OPMessage ();
	virtual ~ OPMessage ();
	
	virtual void StoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	
public:
	virtual void SetMethod (OPAtomIn name);
	virtual void SetMethod (OPStringIn name);
	virtual void SetMethod (const LatinChar* name);
	
	virtual void SetTarget (OPTargetIn target);
	virtual Bool Send (OPObjectIn dest);
	virtual void Clear ();
	
	//	The message has to handle the row data input/output, the other data types
	//	are being handled by the super-class.
	
	virtual void StoreData (const void* data, Card32 size);	/* OVERRIDDEN */
	virtual void RestoreData (void* data, Card32 size);		/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPMessage, OPStorage);
};

/****************************************************************/

inline const OPMessageRef&
OPMessageRef::operator - () const
{
	OP_ASSERT (this->object);
	((OPMessage*)(this->object))->Clear ();
	return *this;
}

inline const OPMessageRef&
OPMessageRef::operator - (OPAtomIn name) const
{
	OP_ASSERT (this->object);
	((OPMessage*)(this->object))->SetMethod (name);
	return *this;
}

/****************************************************************/

#endif /* _OPaC_MESSAGE_H_ */
