/*
 *	tablist.h
 *
 *	The Tab List class provides a view which will draw the top
 *	part of a tabbed view.
 *
 *	(C) Copyright 1994-1997, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_TABLIST_H_
#define	_OPaC_TABLIST_H_

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

OP_INTERFACE (tablist.h);

/****************************************************************/

/*
 *	Button class. There is a more sophisticated view class,
 *	which is providing the same look : the OPaC Cell.
 */

class OP_PUBLIC OPTabList : public OPControl
{
	
protected:
	OPCoord				space;					//	space between tabs
	Count				first;					//	first tab shown [1..n]
	Count				active;					//	active tab (0 = none)
	OPListRef			tabs;					//	collection of views
	Bool				need_update;			//	TRUE => need to update

protected:
	OPTabList ();
	virtual ~ OPTabList ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void ActionOn ();							/* OVERRIDDEN */
	virtual void ActionOff ();							/* OVERRIDDEN */
	virtual void Action (Bool clicked);					/* OVERRIDDEN */

	virtual Count FindTab (OPEventIn event);
	virtual void ComputeWidths (OPCoord width[]);
	
	virtual void SetTabsParent (OPViewIn view);
	virtual void UpdateTabs ();
	virtual void DrawTab (OPCoord x, OPCoord y, OPCoord dx,
						  OPGrailPortIn port, OPViewIn view, Bool enlarged);
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool SetTabs (OPListIn tabs);
	virtual Bool SetSpace (OPCoord space);
	virtual Bool SetFirstTab (Count first);
	virtual Bool SetActiveTab (Count active);
	
	virtual Bool    GetTabs (OPListOut tabs) const;
	virtual OPCoord ReturnSpace () const;
	virtual Count   ReturnFirstTab () const;
	virtual Count   ReturnActiveTab () const;
	
	//	Inherited methods :
	
	virtual Bool SetWindow (OPWindowIn window);			/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	virtual void ButtonPress (OPEventIn event);			/* OVERRIDDEN */
	virtual void ButtonRelease (OPEventIn event);		/* OVERRIDDEN */
	virtual void EnterNotify (OPEventIn event);			/* OVERRIDDEN */
	virtual void MotionNotify (OPEventIn event);		/* OVERRIDDEN */
	virtual void LeaveNotify (OPEventIn event);			/* OVERRIDDEN */
	virtual void KeyDown (OPEventIn event);				/* OVERRIDDEN */
	virtual void KeyUp (OPEventIn event);				/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPTabList, OPControl);
};

/****************************************************************/

#endif /* _OPaC_TABLIST_H_ */

