/*
 *	sd_colman.cxx
 *
 *	OPaC GRAIL color manager for X11 systems
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

// Modified by Laurent Bovet 26.11.97

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "sd_interface.h"

#include <OPaC/grail.h>
#include <OPaC/new.h>

/****************************************************************/

OP_MORE_IMPLEMENTATION (OPGrailPort);

/****************************************************************/

/*
 *	Initialize a color manager. Determine if it will have to
 *	use an indexed or a true color model.
 */	

OPColorMan::OPColorMan ()
{
	//	First, put every field into a known state (zero, zero, zero...)
	
	this->info = OP_ALLOC_FROM_ZONE (OPaC_sys_zone, OPaC_ColorMan);
	this->info->xcolormap = 0;
	
	this->depth = 0;
	
	this->pal_size  = 0;						//	no palette entries
	this->pal_gray  = 0;
	this->pal_cube  = 0;
	this->pal_named = 0;
	this->pal_entry = 0;
	
	this->cube_pos  = 0;
	
	this->named_num   = 0;						//	no named entries
	this->named_pos   = 0;
	this->named_entry = 0;
	
	this->color_model = OP_COLOR_MODEL_NONE;
	this->gray_model  = OP_GRAY_MODEL_MONO;
	
	// Hypothesis: we have a true color display (i.e. 24 bitplanes)

	this->color_model = OP_COLOR_MODEL_GOOD;
		
	this->gray_model  = OP_GRAY_MODEL_GOOD;
		
	this->depth = 24;

	this->ComputeNearestColor ();
}



/* 
 *	Clean up the color manager.
 */

OPColorMan::~OPColorMan ()
{
	if (this->info) {
		OP_FREE_FROM_ZONE (OPaC_sys_zone, this->info);
	}
	
	if (this->pal_entry) {
		OP_FREE_FROM_ZONE (OPaC_sys_zone, this->pal_entry);
	}
}



/*
 *	Return the pixel value for the RGB components. This
 *	will be used to get a true color pixel representation.
 */

OPColorPixel
OPColorMan::MakeTrueColor (Card8 r, Card8 g, Card8 b) const
{
	OP_METHOD (MakeTrueColor, 3);
	OP_ARG (2, Card8, r);
	OP_ARG (3, Card8, g);
	OP_ARG (4, Card8, b);	

	XColor xcolor;

	xcolor.red   = r << 8; // X color components are 16 bit values
	xcolor.green = g << 8;
	xcolor.blue  = b << 8;

	XAllocColor( xdpy, XDefaultColormap(xdpy, 0), &xcolor );

	return (OPColorPixel)(xcolor.pixel);
}



/*
 *	Return the color components from a pixel value.
 */

Bool
OPColorMan::FindColor (OPColorPixel p, Card8Out r, Card8Out g, Card8Out b) const
{
	OP_METHOD (FindColor, 4);
	OP_ARG (1, OPColorPixel, p);
	OP_ARG (2, Card8Out, r);
	OP_ARG (3, Card8Out, g);
	OP_ARG (4, Card8Out, b);
	
	XColor xcolor;

	xcolor.pixel = (unsigned long) p;

	XQueryColor( xdpy, XDefaultColormap(xdpy, 0), &xcolor );

	r = xcolor.red >> 8;
	g = xcolor.green >> 8;
	b = xcolor.blue >> 8;
	
	return TRUE;
}

