/*
 *	sd_debug.cxx
 *
 *	This file implements the debugging tools needed by the OPaC
 *	project files.
 *
 *	This file contains system specific stuff, but it is shared
 *	between every OPaC port.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

//	MS-Windows needs some specific header files :

#ifdef _WINDOWS
#include <string.h>
#include <malloc.h>
#include "sd_interface.h"
#endif /* _WINDOWS */

#include <OPaC/string.h>
#include <OPaC/new.h>

/*
 *	This global function will be called in order to kill the
 *	application after a fatal condition has been detected. It
 *	does NO cleansing at all.
 */

volatile void
OPaC_ErrorAbort (const LatinChar* message, int line, const LatinChar* file)
{
	OPaC_ErrorPrint (message, line, file);
	abort ();
	for (;;);
}

/*
 *	Display an error message for debugging purposes. This may
 *	open a message window or just write to "stderr".
 */

void
OPaC_ErrorPrint (const LatinChar* message, int line, const LatinChar* file)
{
#ifdef _WINDOWS
	Size len = strlen (file) + strlen (message) + 40;
	LatinChar* buffer = (LatinChar*)_alloca ((size_t)(len));
	sprintf (buffer, "%s:%d:error: %s\n", file, line, message);
	MessageBox (NULL, buffer, "Error", MB_OK);
#else /* ! _WINDOWS */
	fprintf (stderr, "%s:%d:error: %s\n", file, line, message);
#endif /* ! _WINDOWS */
}



/*
 *	Display an error message for debugging purposes. This may
 *	open a message window or just write to "stderr".
 */

void
debuggerAboutMemory (const void* ptr)
{
	OPMemZone* zone = 0;

	OPStringRef type;
	OPStringRef file;
	Count       line;
	char        message[300];
	
	if ( (OPMemZone::FindZone (ptr, zone))
	  && ((ptr = zone->FindBasePointer (ptr)) != 0)
	  && (zone->AboutBasePointer (ptr, type, line, file)) ) {
		
		//	Memory stored in "zone".

		LatinChar type_name[50];
		LatinChar file_name[200];
		type->GetStringCopy (type_name, 50);
		file->GetStringCopy (file_name, 200);

		sprintf (message, "PTR=%lx/TYPE=%s in %d:%s", (long)(ptr), type_name, line, file_name);
		
	} else {
		
		//	Unknown memory chunk...

		sprintf (message, "unknown address");
	}
	
#ifdef _WINDOWS
	MessageBox (NULL, message, "Debugger: About Memory", MB_OK);
#else /* ! _WINDOWS */
	fprintf (stderr, "Debugger: About Memory %s\n", message);
#endif /* ! _WINDOWS */
}
