/*
 *	sd_interface.h
 *
 *	This file defines system dependent includes...
 *  Interface for Unix/X11.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

// Modified by Laurent Bovet 18.11.98

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_TYPES_H_
#include <OPaC/types.h>
#endif

// X window
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

// Unix
#include <sys/time.h>

//***************************************************************************//

// Some symbols defined by Xlib are also used by OPaC
// let''s clean that and define new ones for further
// use.

#undef Bool           // X Bool is int
#define xBool int
#define Bool Card8    // OPaC Bool is Card8

// in X the following symbols are constants, in OPaC they are method names.
// So are they redefined with an x before

#undef ButtonPress
#define xButtonPress   4

#undef ButtonRelease
#define xButtonRelease 5

#undef MotionNotify
#define xMotionNotify  6

#undef EnterNotify
#define xEnterNotify   7

#undef LeaveNotify
#define xLeaveNotify   8


//***************************************************************************//

// X Window property names
// These are used to identify an OPaC window and to retrieve the associated
// OPaC structure.

#define OP_XPROP_WIN_INST    "OPaC_WIN_INST"
// Stores a pointer to the OPGrailWindow object

#define OP_XPROP_WIN_ID      "OPaC_WIN_ID"      
// Stores a string identifying the window (should contain PID and OPaC win ID)


// Pointer to display
extern Display* xdpy; 

/* This is a global variable. It must be accessible */
/* by a lot of methods. */
/* This induces that an OPaC application can use */
/* one display (according to OPaC portability specs) */



/*
 *	Private display context definitions.
 */

struct OPaC_GrPort 
{
	Drawable xdrawable;    // The Drawable (window or pixmap) that implement the port
	GC xgc;                // The Graphics Context attached to the port
};

/*
 *	Private pixmap definitions.
 */

struct OPaC_GrPixmap
{
	Pixmap xpixmap;
};

// Prefix for the windows'' IDs

#define	GLOBNAME_ID		"OPaC:Grail:1:"

/*
 *	Private window definitions.
 */

struct OPaC_GrWindow
{
	Window xwindow;
	static Count counter;          /* window counter (is it really needed?) */	
	static char globname[64];      /* global window name */
	LatinChar wclass_name[64];		 /* window class name (unique) */
	XRectangle xredraw_rec;         /* the clip rect when redraw */
};


/*
 *	Private color manager definitions.
 */

struct OPaC_ColorMan
{
	Colormap xcolormap;
};



/*
 *	Private cursor definitions.
 */

struct OPaC_GrCursor
{
	Cursor xcursor;
};



/*
 *	Private font definitions.
 */

struct OPaC_GrFont
{
	Font xfont;
};
