/*
 *	arrayit.cxx
 *
 *	The Array needs a dedicated Iterator class.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/arrayit.h>
#include <OPaC/array.h>
#include <OPaC/new.h>

OP_IMPLEMENT_CLASS (OPArrayIterator);

OPArrayIterator::OPArrayIterator ()
{
}

OPArrayIterator::~OPArrayIterator ()
{
}

void
OPArrayIterator::StoreInstance (OPStorageIn storage)
{
	super::StoreInstance (storage);
	storage << OPStorageSync::mark;
}

void
OPArrayIterator::RestoreInstance (OPStorageIn storage)
{
	super::RestoreInstance (storage);
	storage >> OPStorageSync::mark;
}


void
OPArrayIterator::NotifyInsert (Count pos, Count num)
{
	if (pos > this->index) return;
	this->index += num;
}


void
OPArrayIterator::NotifyRemove (Count pos, Count num)
{
	if (pos > this->index) return;
	if (pos + num >= this->index) {
		this->index -= num;
	} else {
		this->index = pos;
		this->did_change = TRUE;
	}
}


void
OPArrayIterator::Delete ()
{
	super::Delete ();
}

Bool
OPArrayIterator::First ()
{
	this->did_change |= (this->index == 0) ? FALSE : TRUE;
	this->index = 0;
	
	return TRUE;
}

Bool
OPArrayIterator::Last ()
{
	Count num = this->source->TotalCount ();
	
	if (num == 0) return FALSE;
	
	this->did_change |= (this->index == num-1) ? FALSE : TRUE;
	this->index = num-1;
	
	return TRUE;
}

Bool
OPArrayIterator::Next (Count move)
{
	Count num = this->source->TotalCount ();
	
	if (num == 0) return FALSE;
	
	if (this->index < num-1) {
		this->index++;
		this->did_change = TRUE;
		return TRUE;
	}
	
	return FALSE;
}

Bool
OPArrayIterator::Prev (Count move)
{
	if (this->index == 0) return FALSE;
	
	this->index--;
	this->did_change |= TRUE;
	
	return TRUE;
}

Bool
OPArrayIterator::Get (OPObjectOut object, Count index)
{
	this->did_change = FALSE;
	
	if (index == 0) {
		OPArrayRef array = this->source;
		return array->Get (this->index, object);
	}
	
	return FALSE;
}
