/*
 *	objrootprovider.cxx
 *
 *	The Object Root Provider helps browsing object data fields. It
 *	implements the root object...
 *	
 *	(C) Copyright 1997-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/icon.h>
#include <OPaC/lookman.h>
#include <OPaC/new.h>
#include <OPaC/objectbrowser.h>
#include <OPaC/string.h>

/****************************************************************/

OP_IMPLEMENT_CLASS (OPObjRootProvider);

/****************************************************************/


OPObjRootProvider::OPObjRootProvider ()
{
	OP_METHOD (OPObjRootProvider, 0);
}

OPObjRootProvider::~OPObjRootProvider ()
{
	OP_METHOD (~OPObjRootProvider, 0);
}

void
OPObjRootProvider::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
	
	storage << this->object;
	storage << OPStorageSync::mark;
}

void
OPObjRootProvider::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::RestoreInstance (storage);
	
	storage >> this->object;
	storage >> OPStorageSync::mark;
}


void
OPObjRootProvider::Initialise (OPObjectIn object)
{
	OP_METHOD (Initialise, 1);
	OP_ARG (1, OPObjectIn, object);
	
	this->object = object;
}

Count
OPObjRootProvider::ReturnNumObjects ()
{
	OP_METHOD (ReturnNumObjects, 0);
	return 1;
}


Bool
OPObjRootProvider::GetObject (Count index, OPObjectOut object)
{
	OP_METHOD (GetObject, 2);
	OP_ARG (1, Count, index);
	OP_ARG (2, OPObjectOut, object);
	
	object = this->object;
	
	return TRUE;
}

Bool
OPObjRootProvider::GetText (Count index, OPStringOut text)
{
	OP_METHOD (GetText, 2);
	OP_ARG (1, Count, index);
	OP_ARG (2, OPStringOut, text);
	
	if (this->object.IsValid ()) {
		
		static OPAtomRef user_friendly_name = "UserFriendlyName";
		OPTextRef name;
		
		if ( (this->object->GetAttribute (user_friendly_name, name))
		  && (name->IsKindOf (OP_RUNTIME (OPText))) ) {
			text = name->ReturnString ();
			return TRUE;
		}
		
		if (this->object->IsKindOf (OP_RUNTIME (OPApp))) {
			text = "Application";
			return TRUE;
		}
		if (this->object->IsKindOf (OP_RUNTIME (OPMenu))) {
			text = "Menu";
			return TRUE;
		}
		if (this->object->IsKindOf (OP_RUNTIME (OPWindow))) {
			text = "Window";
			return TRUE;
		}
		if (this->object->IsKindOf (OP_RUNTIME (OPDataSlot))) {
			text = "DataSlot";
			return TRUE;
		}
		
		text = "Object";
		return TRUE;
	}
	
	return FALSE;
}

Bool
OPObjRootProvider::GetLargeIcon (Count index, OPLookIconOut icon)
{
	OP_METHOD (GetLargeIcon, 2);
	OP_ARG (1, Count, index);
	OP_ARG (2, OPLookIconOut, icon);
	
	if (this->object.IsValid ()) {
		if (this->object->IsKindOf (OP_RUNTIME (OPApp))) {
			icon = OP_ICON_OBJ_APP;
			return TRUE;
		}
		if (this->object->IsKindOf (OP_RUNTIME (OPMenu))) {
			icon = OP_ICON_OBJ_MENU;
			return TRUE;
		}
		if (this->object->IsKindOf (OP_RUNTIME (OPWindow))) {
			icon = OP_ICON_OBJ_WINDOW;
			return TRUE;
		}
		if (this->object->IsKindOf (OP_RUNTIME (OPDataSlot))) {
			icon = OP_ICON_OBJ_SLOT;
			return TRUE;
		}
		icon = OP_ICON_CUBE_NORMAL;
		return TRUE;
	}
	return FALSE;
}

Bool
OPObjRootProvider::GetSmallIcon (Count index, OPLookIconOut icon)
{
	OP_METHOD (GetSmallIcon, 2);
	OP_ARG (1, Count, index);
	OP_ARG (2, OPLookIconOut, icon);
	
	return FALSE;
}

Bool
OPObjRootProvider::GetMarkIcon (Count index, OPLookIconOut icon)
{
	OP_METHOD (GetMarkIcon, 2);
	OP_ARG (1, Count, index);
	OP_ARG (2, OPLookIconOut, icon);
	
	if ( (this->object.IsValid ())
	  && (this->object->CountPublicData ()) ) {
		
		icon = OP_ICON_SUBMENU_NORMAL;
		
		return TRUE;
	}
	
	return FALSE;
}

Bool
OPObjRootProvider::EnterSubtree (Count index, OPGenericProviderOut provider)
{
	OP_METHOD (EnterSubtree, 2);
	OP_ARG (1, Count, index);
	OP_ARG (2, OPGenericProviderOut, provider);
	
	if ( (this->object.IsValid ())
	  && (this->object->CountPublicData ()) ) {
		
		OPObjBProviderRef pro = OP_NEW (OPObjBProvider);
		
		pro->Initialise (this->object);
		provider = pro;
		
		return TRUE;
	}
	
	return FALSE;
}

Bool
OPObjRootProvider::ReleaseSubtree (Count index)
{
	OP_METHOD (ReleaseSubtree, 1);
	OP_ARG (1, Count, index);
	
	return TRUE;
}

