/*
 *	stream.cxx
 *
 *	The stream class provides a unified interface enabling the
 *	access to files, resources and pipes.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/stream.h>

/****************************************************************/

OP_IMPLEMENTATION (stream.h);
OP_ABSTRACT_CLASS (OPStream);

/****************************************************************/

/*
 *	Initialise a stream object.
 */

OPStream::OPStream ()
{
	OP_METHOD (OPStream, 0);
}



/*
 *	Destroy a stream object.
 */

OPStream::~OPStream ()
{
	OP_METHOD (~OPStream, 0);
}



/*
 *	Store the stream to an archive. This is quite stupid,
 *	or at least I think so.
 */

void
OPStream::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
}



/*
 *	Restore a stream from an archive. This is quite stupid
 *	too.
 */

void
OPStream::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::RestoreInstance (storage);
}



/*
 *	Generic read from a stream.
 *
 *	Throws :
 *
 *	- OPExcStream (NO_READ). This will be the default behavior
 *	  of every non-overridden Read.
 */

Count
OPStream::Read (void* buffer, Count size)
{
	OP_METHOD (Read, 2);
	OP_ARG (1, void*, buffer);
	OP_ARG (2, Count, size);
	
	THROW (OPExcStream (OPExcStream::NO_READ));
	
	return 0;
}



/*
 *	Generic skip while reading stream.
 *
 *	Throws :
 *
 *	- OPExcStream (NO_READ). This will be the default behavior
 *	  of every non-overridden Skip.
 */

Count
OPStream::Skip (Count size)
{
	OP_METHOD (Skip, 1);
	OP_ARG (1, Count, size);
	
	THROW (OPExcStream (OPExcStream::NO_READ));
	
	return 0;
}



/*
 *	Generic write to a stream.
 *
 *	Throws :
 *
 *	- OPExcStream (NO_WRITE). This will be the default behavior
 *	  of every non-overridden Write.
 */

Count
OPStream::Write (const void* buffer, Count size)
{
	OP_METHOD (Write, 2);
	OP_ARG (1, const void*, buffer);
	OP_ARG (2, Count, size);
	
	THROW (OPExcStream (OPExcStream::NO_WRITE));
	
	return 0;
}



/*
 *	Generic synchronization of a write stream. This should flush
 *	the buffered bytes and write them to destination.
 *
 *	Throws :
 *
 *	- OPExcStream (NO_WRITE). This will be the default behavior
 *	  of every non-overridden Sync.
 */

void
OPStream::Sync ()
{
	OP_METHOD (Sync, 0);
}

