/*
 *	main.cxx
 *
 *	This application is currently a stand-alone interface builder
 *	which will gradually be integrated into the OPaC class library.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "main.h"
#include "appmacro.h"

#include "../ib/builder.h"

#include <OPaC/database.h>
#include <OPaC/dialog.h>
#include <OPaC/event.h>
#include <OPaC/grail.h>
#include <OPaC/new.h>
#include <OPaC/window.h>

#if 0
#include <OPaC/freetype.h>
#endif

#include <OPaC/notmacro.h>
#include <OPaC/pubmacro.h>
#include <OPaC/targetmacro.h>

/****************************************************************/

APP_IMPLEMENT_CLASS (App);

/****************************************************************/

OP_TARGET_ACTOR		(App, Terminate);
OP_TARGET_ACTOR		(App, Minimize);

OP_ACCESSOR (App, X, Real, x);
OP_ACCESSOR (App, Y, Real, y);

OP_PUBLISH_FUNCTION	(App, "Z", ReturnZ, "Z", Real, Real);
OP_PUBLISH_SETVALUE (App, "TTF", SetString, "TTF", OPObject, OPString);

OP_NOTIFY_CHANGE (App, Z, Changed);

/****************************************************************/

App::App ()
{
	this->x = 0;
	this->y = 0;
}

App::~App ()
{
	this->dialog  = 0;
}

extern void OpenFileBrowser ();

/*
 *	Initialise the application.
 */

void
App::Initialise (Bool enable_build)
{
	super::Initialise (enable_build);
	
	//	Load the interface for the dialog and open the corresponding window.
	
	OPBuilderRef builder = this->ReturnBuilder ();
	this->dialog = OP_NEW (OPDialog);
	this->dialog->Interaction ("OPaC Demonstration");
#if 0	
	//	Prepare a child window...
	
	OPWindowRef window;
	this->dialog->GetWindow (window);
	
	OPDialogRef dialog = OP_NEW (OPDialog);
	dialog->AddField ("Main window", window);
	dialog->Interaction ("Another window");
#endif
}

/****************************************************************/

void
App::Terminate ()
{
	OPEvent::PostQuitEvent ();
}

Bool
App::SetString (OPStringIn text)
{
#if 0
	LatinChar buffer[200];
	
	text->GetStringCopy (buffer, sizeof (buffer)-1);
	
	static Bool ok = FALSE;
	static OPFreeTypeRef freetype;
	
	const OPaC_TTGlyph* glyph;
	
	if (ok == FALSE) {
		freetype = OP_NEW (OPFreeType);
		freetype->OpenFace ("times.ttf", 10);
		ok = TRUE;
	}
	
	OPWindowRef window;
	OPGrailPortRef grail;
	OPRect box (0,0,200,20);
	this->dialog->GetWindow (window);
	grail = window->ReturnGrailPort ();
	grail->BeginDraw (box);
	grail->SetForeColor (OPColorMan::ReturnSystemColorMan ()->FindColor (1.0));
	grail->FillRect (box);
	
	OPColorPixel p4 = OPColorMan::ReturnSystemColorMan ()->FindColor (0.0);
	OPColorPixel p3 = OPColorMan::ReturnSystemColorMan ()->FindColor (0.3);
	OPColorPixel p2 = OPColorMan::ReturnSystemColorMan ()->FindColor (0.6);
	OPColorPixel p1 = OPColorMan::ReturnSystemColorMan ()->FindColor (0.8);
	
	int x = 10;
	int y = 15;
	
	Card8* ptr = (Card8*)buffer;
	while (*ptr && (freetype->GetGrayBitmap (*ptr, glyph)) ) {
		
		int ox = x + glyph->x_bearing;
		int oy = y - glyph->y_max;
		
		for (int y = 0; y < glyph->dy; y++) {
			for (int x = 0; x < glyph->dx; x++) {
				switch (glyph->pixmap[x+y*glyph->p_width]) {
					case 4:
						grail->DrawDot (x+ox, y+oy, p4);
						break;
					case 3:
						grail->DrawDot (x+ox, y+oy, p3);
						break;
					case 2:
						grail->DrawDot (x+ox, y+oy, p2);
						break;
					case 1:
						grail->DrawDot (x+ox, y+oy, p1);
						break;
				}
			}
		}
		x += glyph->x_advance;
		ptr++;
	}
	grail->EndDraw ();
	grail->Flush ();
	
//	freetype->CloseFace ();
#endif
	return TRUE;
}

void
App::Minimize ()
{
	int num   = this->window_num;
	int index = 0;

	while (num--) {
		OPWindowRef window;
		if (this->window_list->Get (index++, window)) {
			window->ShowReduced ();
		}
	}
}

/****************************************************************/

Real
App::ReturnZ () const
{
	return this->x * this->y;
}

