/*
 *	connectdlg.h
 *
 *	Provide classes needed to build a connection dialog (in order
 *	to connect source and target objects).
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _CONNECTDLG_H_
#define _CONNECTDLG_H_

#include <OPaC/objectbrowser.h>
#include <OPaC/view.h>

/****************************************************************/

APP_EXPORT_CLASS (OPConBaseDialog);
APP_EXPORT_CLASS (OPConActorDialog);
APP_EXPORT_CLASS (OPConFieldDialog);
APP_EXPORT_CLASS (OPConBrowseDialog);

APP_EXPORT_CLASS (OPConnectDialog);

/****************************************************************/

APP_DEFINE_REF (OPConBaseDialog, OPView);
APP_DEFINE_REF (OPConActorDialog, OPConBaseDialog);
APP_DEFINE_REF (OPConFieldDialog, OPConBaseDialog);
APP_DEFINE_REF (OPConBrowseDialog, OPConFieldDialog);

APP_DEFINE_REF (OPConnectDialog, OPObject);

/****************************************************************/

/*
 *	Base connection view : it builds two lists with souce and
 *	target elements which can then be connected.
 */

class OPConBaseDialog : public OPView
{
protected:
	OPObjectRef			source;
	OPObjectRef			target;
	
	OPDBScrollListRef	source_list;
	OPDBScrollListRef	target_list;
	OPContainerRef		source_container;
	OPContainerRef		target_container;
	
protected:
	OPConBaseDialog ();
	virtual ~ OPConBaseDialog ();
	
	virtual void CreateSourceList (OPCoord x, OPCoord y, OPCoord dx, OPCoord dy) = 0;
	virtual void CreateTargetList (OPCoord x, OPCoord y, OPCoord dx, OPCoord dy) = 0;
	
public:
	virtual void Initialise (OPObjectIn src, OPObjectIn dst);
	virtual void Connect () = 0;
	
	APP_DEFINE_CLASS (OPConBaseDialog, OPView);
};

/****************************************************************/

/*
 *	Specific implementation of a connection between an event
 *	and an actor.
 */

class OPConActorDialog : public OPConBaseDialog
{
	
protected:
	OPConActorDialog ();
	virtual ~ OPConActorDialog ();
	
	virtual void CreateSourceList (OPCoord x, OPCoord y, OPCoord dx, OPCoord dy);
	virtual void CreateTargetList (OPCoord x, OPCoord y, OPCoord dx, OPCoord dy);
	
public:
	virtual void Connect ();
	
	APP_DEFINE_CLASS (OPConActorDialog, OPConBaseDialog);
};

/****************************************************************/

/*
 *	Specific implementation of a connection between two data
 *	fields (source and target).
 */

class OPConFieldDialog : public OPConBaseDialog
{
	
protected:
	Bool				use_display_actor;
	
protected:
	OPConFieldDialog ();
	virtual ~ OPConFieldDialog ();
	
	virtual void CreateSourceList (OPCoord x, OPCoord y, OPCoord dx, OPCoord dy);
	virtual void CreateTargetList (OPCoord x, OPCoord y, OPCoord dx, OPCoord dy);
	
public:
	virtual void Connect ();
	virtual void ConnectToDisplayActor (Bool yes);
	
	APP_DEFINE_CLASS (OPConFieldDialog, OPConBaseDialog);
};

/****************************************************************/

/*
 *	Specific implementation of a connection between two data
 *	fields (source and target, but the target originates from
 *	browsing).
 */

class OPConBrowseDialog : public OPConFieldDialog
{
	
protected:
	OPObjectBrowserRef	target_browser;
	
protected:
	OPConBrowseDialog ();
	virtual ~ OPConBrowseDialog ();
	
public:
	virtual void Initialise (OPObjectIn src, OPObjectIn dst);	/* OVERRIDDEN */
	virtual void Connect ();									/* OVERRIDDEN */
	virtual void IconsChanged (OPObjectIn object, Card32 id);
	
	APP_DEFINE_CLASS (OPConBrowseDialog, OPConFieldDialog);
};

/****************************************************************/

/*
 *	Implementation of a full dialog for connecting a source
 *	and a target. Provides a window, buttons, lists, etc...
 */

class OPConnectDialog : public OPObject
{
protected:
	OPWindowRef			 window;
	OPConActorDialogRef	 actors;
	OPConFieldDialogRef	 fields;
	OPConBrowseDialogRef browse;
	OPTabViewRef		 tab_view;
	OPButtonRef			 button_connect;
	OPButtonRef			 button_cancel;
	
protected:
	OPConnectDialog ();
	virtual ~ OPConnectDialog ();
	
	virtual void Kill ();
	
public:
	virtual void Initialise (OPObjectIn src, OPObjectIn dst);
	
	virtual void Connect ();
	virtual void Cancel ();
	
	APP_DEFINE_CLASS (OPConnectDialog, OPObject);
};

/****************************************************************/

#endif /* _CONNECTDLG_H_ */
